#include "include.h"

/** V A R I A B L E S ********************************************************/
#pragma udata
// CPU configuration
// PLLDIV = 5 (20MHz). full speed USB, CPU@48MHz, clocked from external crystal
#pragma config PLLDIV = 5, CPUDIV = OSC1_PLL2, USBDIV = 2
#pragma config FOSC = HSPLL_HS
#pragma config IESO = OFF, PWRT = ON
#pragma config BOR = OFF, VREGEN = ON, WDT = OFF
#pragma config MCLRE = ON, LPT1OSC = OFF, PBADEN = ON
#pragma config STVREN = ON, LVP = OFF
#pragma config DEBUG = OFF
#pragma config CP0 = OFF, CP1 = OFF, CPB = OFF
#pragma config WRT0 = OFF, WRT1 = OFF, WRTB = OFF
#pragma config WRTC = OFF, EBTR0 = OFF, EBTR1 = OFF, EBTRB = OFF

WORD b;
byte gfxReg[5];
	VOLINFO vi;
	DIRINFO di;
	DIRENT de;

//init sd card data structures (includeing 2x512k ram sections)
#pragma udata MSD=0x400
far unsigned char sector[SECTOR_SIZE];
#pragma udata MSDT=0x100
far unsigned char sector2[SECTOR_SIZE];

	//A0 - LAT
	//A1 - SCK
	//A2 - SDO
	//A3 - SDI (error checking input...)
//#define GFX_LAT  //dont confuse this with the 595 'lat' signal, this is the PIC LAT for data output...
#define GFX_DO	LATAbits.LATA2
//#define GFX_DI  //for eventual POST (self test) by reading bytes in and out of 595s....
#define GFX_CLK	LATAbits.LATA1
#define GFX_CS  LATAbits.LATA0



/** D E C L A R A T I O N S **************************************************/
#pragma code
void _gfxOutIO(byte val,byte pin)
{
	GFX_DO=0;
	if(val&pin) GFX_DO=1;//if bit set, data to 1....
	GFX_CLK=1;//bump the clock
	GFX_CLK=0;
	GFX_DO=0;
}


void gfxOut(byte *DB)
{
	byte i;
	GFX_CLK=0; //LATBbits.LATB2=0; //clock

	//ground latchPin and hold low for as long as you are transmitting
    GFX_CS=0; //LATBbits.LATB0=0;

	//start with last bit - byte 5, bit 1..
	//clock in bits
	for(i=5;i>0;i--){
		_gfxOutIO(DB[i-1],0b00000001);
		_gfxOutIO(DB[i-1],0b00000010);
		_gfxOutIO(DB[i-1],0b00000100);
		_gfxOutIO(DB[i-1],0b00001000);
		_gfxOutIO(DB[i-1],0b00010000);
	}	

    //return the latch pin high to signal chip that it 
    //no longer needs to listen for information
    GFX_CS=1;//LATBbits.LATB0=1;
}


void main(void)
{
	byte bj,ij,j,k,gfxTempArrayCounter,byteCounter;
	byte gfxTempArray[3];
	byte gfxRegCounter;
	word wordCounter;
//	BYTE c=0;
	uint32_t pstart, psize, i;
	uint8_t pactive, ptype;
	byte DPTfile[12];
	uint32_t cache;
	uint8_t *p;
	FILEINFO fi;
	uint32_t r;
	uint16_t c;
	word di_currententry=0;
	byte di_repop=0; 
	//A0-5, I/O
	//C0-2, I/O
	//C4-5, usb
	//c6, usb power detect
	//c7, sdo
	//b0, sdi
	//b1, sck
	//b4, cd
	//b5, cs

	//ADC off, all pins to input/off
	ADCON1=0b00001111; //reset AN0 and references...
	LATA=0x00;
	LATB=0x00;
	LATC=0x00;
	TRISA=0xFF;
	TRISB=0xFF;
	TRISC=0xFF;
	
	//setup port A for gfx IO , RB3 as Input
	//A0 - LAT
	//A1 - SCK
	//A2 - SDO
	//A3 - SDI (error checking input...)
	TRISA=0b00001000;
	
	//setup port B&C for SD card IO
	//b5,1 as output
	//b4,0 as input
	TRISB=0b00010001;
	//c7 as output
	//c6 as input
	TRISC=0b01111111;

	//init SPI
	SSPCON1 = 0b00100010;			// SSPEN bit is set, SPI in master mode, FOSC/4, 
							//   IDLE state is low level
	PIR1bits.SSPIF = 0;
	SSPSTATbits.CKE = 1; 	// Transmit data on rising edge of clock
	SSPSTATbits.SMP = 0;	// Input sampled at middle of data output time

	//init sd card
	SDInit();
	//init MMC mode
	bj=memCardInit();
	for(c=0;c<512;c++) sector[c]=0;

	//init fat
	// Obtain pointer to first partition on first (only) unit
	//pstart = DFS_GetPtnStart(0, sector, 0, &pactive, &ptype, &psize);
	pstart = DFS_GetPtnStart(0, sector, 0);
	//if (pstart == 0xffffffff) { }	//printf("Cannot find first partition\n");

	r=DFS_GetVolInfo(0, sector, pstart, &vi); //{}//printf("Error getting volume information\n");}
	//watch vi for volume info!

	// Directory enumeration test
	di.scratch = sector2;
//	r=DFS_OpenDir(&vi, '/', &di);//	printf("Error opening root directory\n");

	while (1){
//		r=DFS_GetNext(&vi, &di, &de);
//		if(r==1)
//		{
//			r=DFS_OpenDir(&vi, '/', &di);
//			r=DFS_GetNext(&vi, &di, &de);
//		}

		if(di_repop==1)
		{
			r=DFS_OpenDir(&vi, '/', &di);
			wordCounter=1;
			while(wordCounter)
			{
				r=DFS_GetNext(&vi, &di, &de);
				if(di.currententry==di_currententry) break;
			}
			di_repop=0;
		}

		r=DFS_GetNext(&vi, &di, &de);
		if(r==1)
		{
			r=DFS_OpenDir(&vi, '/', &di);
			r=DFS_GetNext(&vi, &di, &de);
		}

		if (de.name[0] && 
		( 
			((de.name[8]==0x44) && (de.name[9]==0x50) && (de.name[10]==0x54)) 
		|| 
			((de.name[8]==0x64) && (de.name[9]==0x70) && (de.name[10]==0x74)) 
		)
		){
			//open first file with .dpt extension
			r=DFS_OpenFile(&vi, de.name, DFS_READ, sector, &fi);//error opening file 
			//printf("file: '%-11.11s'\n", de.name);

			gfxTempArrayCounter=0;
			gfxRegCounter=0;

			while(!(fi.filelen==fi.pointer))//repeat till end of file
			{	
				word sectorBytes,dly;
				

				sectorBytes=fi.filelen-fi.pointer; //calculate the number of bytes in the current sector to read....
				if(sectorBytes>512) sectorBytes=512;
				r=DFS_ReadFile(&fi, sector2, sector, &i, SECTOR_SIZE);
				//set flags that the scratch buffer needs to be repopulated.
				di_repop=1;
				di_currententry=di.currententry;
			
				for(wordCounter=0;wordCounter<sectorBytes;wordCounter++)//needs to be length of last sector read.....
				{
					byte temp, tempcount;
	
					switch(sector[wordCounter])
					{
						case 0x20://space
						case 0x0d: //next line is break, finish up AtoI here, then break on 0x0a...
								//multiply each value by 10*place to get final byte value
								temp=0;
								for(tempcount=0; tempcount<gfxTempArrayCounter; tempcount++){
									temp*=10;
									temp+=gfxTempArray[tempcount];
								}
								gfxReg[gfxRegCounter]=temp;
								gfxTempArrayCounter=0;
								gfxRegCounter++;
							break;
					//	case 0x0d:  //break is comming....
						case 0x0a:
							//repeat x (5) times till end of line 0x0d 0x0a (13/10) - wrong format...error
							//end of line
							gfxRegCounter=0;
							//put data on display
							gfxOut(gfxReg);
							//pause
							for(temp=0; temp<50; temp++) DelayMs(200);
							break;
						default:
							if(0x2f<sector[wordCounter] && sector[wordCounter]<0x3a)
							{
								//read till end of ascii number value (max 3 digit/255/byte) - 0x20 (30) - out of range, no space....error
								//subtract 0x30 (48) from each digit to get decimal value (set bits 5,4 to 0...)
								gfxTempArray[gfxTempArrayCounter]=(sector[wordCounter]&~0x30);
								gfxTempArrayCounter++;
							}
							break;
			
					}
				}

			}

		}
	}

	//number of bytes read is now in i.....
	//on first pass we dont get the string copied into sector, sector2 instead...

		
	//if straddle sector & more byte remain in fi, then get next sector.
//	r=DFS_ReadFile(&fi, sector2, sector, &i, SECTOR_SIZE);
//	b=fi.filelen; //bytesretrievedfromreadfile.....;
while(1);
	
	//open next file and repeat....

}//end main

