#ifndef _HA_LAYER_
#define _HA_LAYER_
#include "include.h"
//#include "sdfat\typedefs.h"
//#include "sdfat\FATLib.h"

/**
 * Hardware port/pin definition
 **/

#define SD_SPI_IF (PIR1bits.SSPIF)

#define SD_SSPBUF (SSPBUF)

#define SD_CSn_DIR	TRISB
#define SD_CSn_OUT	LATB
#define SD_CSn_PIN	0b00100000 //0x08//5

#define SD_CD_DIR	TRISB
#define SD_CD_IN	PORTB
#define SD_CD_PIN	0b00010000 //0x10 //4

//#define SD_WP_DIR	TRISD
//#define SD_WP_IN	PORTD
//#define SD_WP_PIN	0x80

/* Memory card sector size */
#define SECTOR_SIZE 512

/**
 * MMC/SD card SPI mode commands
 **/
#define CMD0 0x40	// software reset
#define CMD1 0x41	// brings card out of idle state
#define CMD2 0x42	// not used in SPI mode
#define CMD3 0x43	// not used in SPI mode
#define CMD4 0x44	// not used in SPI mode
#define CMD5 0x45	// Reserved
#define CMD6 0x46	// Reserved
#define CMD7 0x47	// not used in SPI mode
#define CMD8 0x48	// Reserved
#define CMD9 0x49	// ask card to send card speficic data (CSD)
#define CMD10 0x4A	// ask card to send card identification (CID)
#define CMD11 0x4B	// not used in SPI mode
#define CMD12 0x4C	// stop transmission on multiple block read
#define CMD13 0x4D	// ask the card to send it's status register
#define CMD14 0x4E	// Reserved
#define CMD15 0x4F	// not used in SPI mode
#define CMD16 0x50	// sets the block length used by the memory card
#define CMD17 0x51	// read single block
#define CMD18 0x52	// read multiple block
#define CMD19 0x53	// Reserved
#define CMD20 0x54	// not used in SPI mode
#define CMD21 0x55	// Reserved
#define CMD22 0x56	// Reserved
#define CMD23 0x57	// Reserved
#define CMD24 0x58	// writes a single block
#define CMD25 0x59	// writes multiple blocks
#define CMD26 0x5A	// not used in SPI mode
#define CMD27 0x5B	// change the bits in CSD
#define CMD28 0x5C	// sets the write protection bit
#define CMD29 0x5D	// clears the write protection bit
#define CMD30 0x5E	// checks the write protection bit
#define CMD31 0x5F	// Reserved
#define CMD32 0x60	// Sets the address of the first sector of the erase group
#define CMD33 0x61	// Sets the address of the last sector of the erase group
#define CMD34 0x62	// removes a sector from the selected group
#define CMD35 0x63	// Sets the address of the first group
#define CMD36 0x64	// Sets the address of the last erase group
#define CMD37 0x65	// removes a group from the selected section
#define CMD38 0x66	// erase all selected groups
#define CMD39 0x67	// not used in SPI mode
#define CMD40 0x68	// not used in SPI mode
#define CMD41 0x69	// Reserved
#define CMD42 0x6A	// locks a block
// CMD43 ... CMD57 are Reserved
#define CMD58 0x7A	// reads the OCR register
#define CMD59 0x7B	// turns CRC off
// CMD60 ... CMD63 are not used in SPI mode


/**
 *	Hardware Abstraction Layer initialization
 **/
void SDInit(void);

/**
 *	checks if there is a memory card in the slot
 *
 *	@return	FALSE		card not detected
 *	@return TRUE		car detected
 **/
boolean detectCard(void);

/**
 *	sends a byte on the SPI to the memory card
 *
 *	@param	inBuf		byte to be sent
 **/
void spi_put(unsigned char inBuf);

/**
 *	reads a byte from the SPI to the memory card
 *
 *	@return	...			byte read
 **/
unsigned char spi_get(void);

/**
 *	sends the command to the the memory card with the 4 arguments and the CRC
 *
 *	@param command		the command
 *	@param argX			the arguments
 *	@param CRC			cyclic redundancy check
 **/
void sendCommand(unsigned char command, unsigned char argA, unsigned char argB,
                 unsigned char argC, unsigned char argD, unsigned char CRC);

/**
 *	waits for the memory card to response
 *
 *	@param expRes		expected response
 *
 *	@return TRUE		card responded within time limit
 *	@return FALSE		card did not response
 **/
boolean cardResponse(unsigned char expRes);

/**
 *	initializes the memory card by brign it out of idle state and 
 *	sets it up to use SPI mode
 *
 *	@return TRUE		card successfully initialized
 *	@return FALSE		card not initialized
 **/
byte memCardInit(void);

/**
 *	Initialize the block lenght in the memory card to be 512 bytes
 *	
 *	return TRUE			block length sucessfully set
 *	return FALSE		block length not sucessfully set
 **/
boolean setBLockLength(void);

/**
 *	read content of the sector indicated by the input and place it in the buffer
 *
 *	@param	sector		sector to be read
 *	@param	*buf		pointer to the buffer
 *
 *	@return	...			number of bytes read
 **/
//unsigned int readSector(unsigned long sector, unsigned char *buf);
uint32_t readSector(uint32_t sector, uint8_t *buf);

/**
 *	writes content of the buffer to the sector indicated by the input
 *
 *	@param	sector		sector to be written
 *	@param	*buf		pointer to the buffer
 *
 *	@return	...			number of bytes written
 **/
//unsigned int writeSector(unsigned long sector, unsigned char *buf);
uint32_t writeSector(uint32_t sector, uint8_t *buf);

// Override media sector functions
#define DFS_ReadSector(unit,buffer,sector,count) readSector(sector,buffer)
#define DFS_WriteSector(unit,buffer,sector,count) writeSector(sector,buffer)

#endif
